package ACL2::AnalisisLisp;

=head1 NAME

ACL2::AnalisisLisp - conversiE<oacute>n de listas y E<aacute>rboles Lisp a versiones Perl

=cut

#
#    Módulo de procesado de listas Lisp.
#    Copyright (C) 2006-2008 Antonio García Domínguez
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use utf8;
use Carp;

use ACL2::Localizacion;

my $lh = ACL2::Localizacion->get_handle();

=head1 FUNCTIONS

Abajo se listan todas las funciones proporcionadas por este mE<oacute>dulo.

=head2 dividir_lista

Uso: ME<eacute>todo estE<aacute>tico pE<uacute>blico.

Responsabilidades: Forma una lista de Perl a partir de una lista (o
E<aacute>tomo) de Lisp.

ParE<aacute>metros: listaLisp: cadena con la lista de Lisp (o un
E<aacute>tomo).

Devuelve: lista de Perl con cadenas para cada elemento de la lista de
Lisp (que tambiE<eacute>n pueden ser listas Lisp).

=cut

sub dividir_lista {
    my ($paquete, $listaLisp) = @_;

    # Nivel 0: lista
    # Nivel 1: elementos de lista, y así sucesivamente
    my $nivel = 0;

    my $tmp;
    my @elems = ();
    while ($listaLisp =~ /( \( | \) | [^\(\)\s\|]+ | \|.*?\| | \s+ )/xg) {

        my $elem = $1;

        $elem eq "(" && do {

            ++$nivel;

            # Si estamos ya dentro de la lista, este
            # paréntesis pertenece al elemento, que es una
            # lista
            if ($nivel > 1) {
                $tmp .= $elem;
            }

            next;
        };

        $elem eq ")" && do {

            --$nivel;

            # Se ha cerrado algún paréntesis que no se ha abierto
            if ($nivel < 0) {
                croak $lh->maketext("Paréntesis cerrando sin correspondiente paréntesis abriendo");
            }

            # Cerramos paréntesis de la lista
            if ($nivel > 0) {

                # Paréntesis cerrando dentro del elemento, que es
                # una lista
                $tmp .= $elem;

                # Se ha cerrado el elemento, que era una lista
                if ($nivel == 1) {
                    push @elems, $tmp;
                    $tmp = "";
                }
            }

            next;
        };

        # Átomo
        if ($nivel <= 1 && $elem !~ /^\s+$/ ) {
            # elemento propio de la lista
            push @elems, $elem;
            $tmp = "";
        } elsif ($nivel > 1) {
            # parte de una sublista
            $tmp .= $elem;
        }

    }

    # No se han cerrado todos los paréntesis
    if ($nivel > 0) {
        croak $lh->maketext("Expresión no equilibrada");
    }

    return @elems;
}


=head2 dividir_arbol

Uso: ME<eacute>todo estE<aacute>tico pE<uacute>blico.

Responsabilidades: Divide una cadena con un E<aacute>rbol formado a
partir de listas Lisp en una lista de referencias a lista y cadenas de
Perl. Muy E<uacute>til para procesar E<oacute>rdenes y argumentos de
E<eacute>stas.

ParE<aacute>metros: cadena - cadena con el E<aacute>rbol Lisp.

Devuelve: Lista con cadenas (en caso de E<aacute>tomo) o referencias a
lista (en caso de sublista) para cada elemento del E<aacute>rbol
original.

=cut

sub dividir_arbol
{
    my ($paquete, $cadena) = @_;

    my @lista = __PACKAGE__->dividir_lista($cadena);
    for (my $i=0; $i < @lista; $i++) {
        if ($lista[$i] =~ /\(.*\)/) {
            my @sublista = __PACKAGE__->dividir_arbol($lista[$i]);
            $lista[$i] = \@sublista;
        }
    }

    return @lista;
}

=head2 analizar_fuente_lisp($paquete, $fuenteLisp)

  Uso: ME<eacute>todo estE<aacute>tico pE<uacute>blico.
  Responsabilidades: Realiza el anE<aacute>lisis sintE<aacute>ctico de toda fuente Lisp.
  Entrada:

=over 2

=item *

fuenteLisp - cadena con el cE<oacute>digo fuente Lisp.

=back

  Salida: Lista de cadenas con cada S-expresiE<oacute>n, correspondientes a
  E<oacute>rdenes de ACL2.

=cut

sub analizar_fuente_lisp
{
    my ($paquete, $fuenteLisp) = @_;

    # Retiramos comentarios
    $fuenteLisp =~ s/;.*/ /gm;

    # Obtenemos la lista correspondiente de ordenes
    my @ordenes = __PACKAGE__->dividir_lista("($fuenteLisp)");

    return @ordenes;
}

1;
